@@ -21,18 +21,22 @@ namespace Tomboy
 		static BonoboUIVerb [] menu_verbs;
 
 		public TomboyApplet (IntPtr raw)
-			: base (raw)
+: base (raw)
 		{
 		}
 
-		public override string IID 
+		public override string IID
 		{
-			get { return "OAFIID:TomboyApplet"; }
+			get {
+				return "OAFIID:TomboyApplet";
+			}
 		}
 
-		public override string FactoryIID 
+		public override string FactoryIID
 		{
-			get { return "OAFIID:TomboyApplet_Factory"; }
+			get {
+				return "OAFIID:TomboyApplet_Factory";
+			}
 		}
 
 		public override void Creation ()
@@ -46,6 +50,7 @@ namespace Tomboy
 			Flags |= PanelAppletFlags.ExpandMinor;
 
 			Add (tray);
+			Tomboy.Tray = tray;
 			OnChangeSize (Size);
 			ShowAll ();
 
@@ -60,9 +65,9 @@ namespace Tomboy
 			SetupMenuFromResource (null, "GNOME_TomboyApplet.xml", menu_verbs);
 		}
 
-		new void SetupMenuFromResource (Assembly asm, 
-						string resource, 
-						BonoboUIVerb [] verbs)
+		new void SetupMenuFromResource (Assembly asm,
+		                                string resource,
+		                                BonoboUIVerb [] verbs)
 		{
 			if (asm == null)
 				asm = GetType ().Assembly;
@@ -95,9 +100,9 @@ namespace Tomboy
 			Tomboy.ActionManager ["ShowAboutAction"].Activate ();
 		}
 
-		protected override void OnChangeBackground (PanelAppletBackgroundType type, 
-							    Gdk.Color                 color, 
-							    Gdk.Pixmap                pixmap)
+		protected override void OnChangeBackground (PanelAppletBackgroundType type,
+		                Gdk.Color                 color,
+		                Gdk.Pixmap                pixmap)
 		{
 			if (tray == null)
 				return;
@@ -139,7 +144,7 @@ namespace Tomboy
 		private static extern IntPtr egg_tray_icon_new (string name);
 
 		public TomboyTrayIcon ()
-			: this (Tomboy.DefaultNoteManager)
+: this (Tomboy.DefaultNoteManager)
 		{
 		}
 
@@ -157,7 +162,7 @@ namespace Tomboy
 			ShowAll ();
 		}
 
-		void ButtonPress (object sender, Gtk.ButtonPressEventArgs args) 
+		void ButtonPress (object sender, Gtk.ButtonPressEventArgs args)
 		{
 			Gtk.Widget parent = (Gtk.Widget) sender;
 
@@ -223,10 +228,12 @@ namespace Tomboy
 		{
 			Tomboy.ActionManager ["QuitTomboyAction"].Activate ();
 		}
-		
+
 		public TomboyTray TomboyTray
 		{
-			get { return tray; }
+			get {
+				return tray;
+			}
 		}
 	}
 }
